import re
from ..conf import settings
from ..helpers import storage, find_static
from django.forms.widgets import flatatt
from django.utils.datastructures import SortedDict
from django.template import Library, Node, TemplateSyntaxError


register = Library()
kw_pat = re.compile(r'^(?P<key>[\w]+)=(?P<value>.+)$')


class ShrinkNode(Node):
    error_message = ''
    endtag = ''
    template = ''

    def __init__(self, parser, token):
        bits = token.split_contents()
        if len(bits) < 2:
            raise TemplateSyntaxError(self.error_message)
        self.destination = bits[1].strip('\'"')
        attrs = SortedDict()
        for bit in bits[2:]:
            m = kw_pat.match(bit)
            if not m:
                raise TemplateSyntaxError(self.error_msg)
            attrs[m.group('key')] = m.group('value')
        self.attrs = flatatt(attrs)
        self.nodelist = parser.parse((self.endtag,))
        parser.delete_first_token()

    def get_prefix(self, path):
        return settings.STATIC_URL

    def get_paths(self, context={}, absolute=False):
        block = self.nodelist.render(context)
        paths = []
        for path in block.replace('\r\n', '\n').split('\n'):
            path = path.strip()
            if path and absolute:
                path = find_static(path)
            if path:
                paths.append(path)
        return paths

    def render(self, context):
        if settings.DEBUG:
            tags = []
            for path in self.get_paths(context):
                tag = self.template.format(prefix=self.get_prefix(path),
                    path=path, attrs=self.attrs)
                tags.append(tag)
            return '\n'.join(tags)
        else:
            path = storage.url(self.destination)
            if settings.SHRINK_TIMESTAMP:
                try:
                    timetamp = storage.modified_time(
                        self.destination).isoformat()
                    path = '%s?%s' % (path, timetamp)
                except Exception:
                    pass
            return self.template.format(prefix='', path=path, attrs=self.attrs)


@register.tag('scripts')
class ScriptNode(ShrinkNode):
    error_message = 'Usage: scripts destination [keyword arguments]'
    endtag = 'endscripts'
    template = '<script src="{prefix}{path}"{attrs}></script>'

    def __repr__(self):
        return '<ScriptNode>'


@register.tag('styles')
class StyleNode(ShrinkNode):
    error_message = 'Usage: styles destination [keyword arguments]'
    endtag = 'endstyles'
    template = '<link rel="stylesheet" href="{prefix}{path}"{attrs}>'

    def get_prefix(self, path):
        if path.endswith('.scss'):
            return settings.SHRINK_SCSS_URL
        return settings.STATIC_URL

    def __repr__(self):
        return '<StyleNode>'




import posixpath
import sys
import urllib
from django.conf import settings
from django.contrib.staticfiles import finders
from django.utils.functional import LazyObject


class LazyStorage(LazyObject):
    def _setup(self):
        self._wrapped = import_string(settings.SHRINK_STORAGE)()
storage = LazyStorage()


def handle_extensions(extensions=('html',)):
    """
    organizes multiple extensions that are separated with commas or passed by
    using --extension/-e multiple times.

    for example: running 'django-admin makemessages -e js,txt -e xhtml -a'
    would result in a extension list: ['.js', '.txt', '.xhtml']

    >>> handle_extensions(['.html', 'html,js,py,py,py,.py', 'py,.py'])
    ['.html', '.js']
    >>> handle_extensions(['.html, txt,.tpl'])
    ['.html', '.tpl', '.txt']
    """
    ext_list = []
    for ext in extensions:
        ext_list.extend(ext.replace(' ','').split(','))
    for i, ext in enumerate(ext_list):
        if not ext.startswith('.'):
            ext_list[i] = '.%s' % ext_list[i]
    return set(ext_list)


def find_static(path):
    normalized_path = posixpath.normpath(urllib.unquote(path)).lstrip('/')
    absolute_path = finders.find(normalized_path)
    if absolute_path:
        return absolute_path.decode(settings.FILE_CHARSET)


def import_string(dot_name):
    """
    Imports an object based on a string.

    :param dot_name: the dot_name name for the object to import.
    :return: imported object
    """
    # force the import dot_name to automatically convert to strings
    if isinstance(dot_name, unicode):
        dot_name = str(dot_name)
    if '.' in dot_name:
        mod_name, attr = dot_name.rsplit('.', 1)
    else:
        return __import__(dot_name)
    # __import__ is not able to handle unicode strings in the fromlist if the
    # mod_name is a package
    if isinstance(attr, unicode):
        attr = attr.encode('utf-8')
    try:
        return getattr(__import__(mod_name, None, None, [attr]), attr)
    except (ImportError, AttributeError):
        # support importing mod_names not yet set up by the parent mod_name (or
        # package for that matter)
        mod_name = '%s.%s' % (mod_name, attr)
        try:
            __import__(mod_name)
        except ImportError, e:
            raise ImportError('Failed to import %s: %s' % (mod_name, e))
        return sys.modules[mod_name]




import os
from os.path import abspath, dirname, join as pjoin
from django.conf import settings


here = abspath(dirname(__file__))
shrinks = pjoin(here, os.pardir, 'shrinks')


SHRINK_TIMESTAMP = True

SHRINK_STORAGE = settings.STATICFILES_STORAGE

SHRINK_CLOSURE_COMPILER = pjoin(shrinks, 'cc', 'compiler.jar')

SHRINK_CLOSURE_COMPILER_COMPILATION_LEVEL = 'SIMPLE_OPTIMIZATIONS'

SHRINK_YUI_COMPRESSOR = pjoin(shrinks, 'yui', 'yuicompressor-2.4.6.jar')

SHRINK_SCSS_URL = '/scss/'




import os
from django.conf import settings
from django.contrib.staticfiles.management.commands.collectstatic import Command as CollectStaticCommand
from django.template.loader import get_template
from shrink.helpers import import_string, handle_extensions
from shrink.base import StyleCompressor, ScriptCompiler
from shrink.templatetags.shrink import ScriptNode, StyleNode
from optparse import make_option
from os.path import isdir, splitext, join as pjoin


def rshrink(node, t):
    """
    Recursive fun
    """
    if isinstance(node, ScriptNode):
        shrink = ScriptCompiler(node, t.name)
        shrink.update()
    elif isinstance(node, StyleNode):
        shrink = StyleCompressor(node, t.name)
        shrink.update()
    if hasattr(node, 'nodelist'):
        for n in node.nodelist:
            rshrink(n, t)


class Command(CollectStaticCommand):
    option_list = CollectStaticCommand.option_list + (
        make_option('--extension', '-e', dest='extensions', default=['html'],
            help=(
                'The file extension(s) to examine for scripts and css '
                '(default: ".html", separate multiple extensions with commas, '
                'or use -e multiple times)'
                ),
            action='append'),
        make_option('--noshrink', action='store_false', dest='shrink',
            default=True, help="Do NOT shrink scripts or css."),
    )
    help = (
        "Collect static files from apps and other locations in a single"
        "location.\nShrinks javascripts and css/scss defined in templates."
        )

    def handle_noargs(self,  **options):
        super(Command, self).handle_noargs(**options)
        if not options.get('shrink'):
            return
        extensions = handle_extensions(options['extensions'])
        templates = set()
        for loader_dot in settings.TEMPLATE_LOADERS:
            loader = import_string(loader_dot)()
            if hasattr(loader, 'get_template_sources'):
                for template_dir in loader.get_template_sources(''):
                    if isdir(template_dir):
                        for (dirpath, dirnames, filenames) in os.walk(template_dir):
                            for f in filenames:
                                if splitext(f)[1] in extensions:
                                    templates.add(get_template(pjoin(dirpath, f)))
        for t in templates:
            rshrink(t, t)




from shrink.conf import settings
from django.conf.urls.defaults import patterns


urlpatterns = []


if settings.DEBUG:
    urlpatterns += patterns('', (
        r'^%s/(?P<path>.*\.scss)$' % settings.SHRINK_SCSS_URL.strip('/'),
        'shrink.views.scss'
        ))




import datetime
import os
import tempfile
from django.conf import settings
from django.core.files.base import ContentFile
from django.utils.encoding import smart_str
from os.path import isdir, getmtime, dirname
from scss import Scss
from shrink.helpers import storage
from subprocess import Popen, PIPE
from tempfile import mkstemp


class Shrink(object):
    already_treated = ''

    def __init__(self, node, template):
        self.node = node
        self.template = template
        self.absolute_paths = self.node.get_paths(absolute=True)

    def therapy(self):
        raise NotImplemented

    def update(self):
        latest = sorted(self.absolute_paths, key=getmtime, reverse=True)[0]
        if (
            not storage.exists(self.node.destination) or
            storage.modified_time(self.node.destination) <
            datetime.datetime.fromtimestamp(getmtime(latest))
            ):
            if storage.exists(self.node.destination):
                storage.delete(self.node.destination) # or else we get next available name
            self.therapy()
        else:
            print self.already_treated


class ScriptCompiler(Shrink):
    already_treated = 'Scripts:\tup-to-date.'

    def therapy(self):
        args = ['java', '-jar', settings.SHRINK_CLOSURE_COMPILER,
                '--compilation_level',
                settings.SHRINK_CLOSURE_COMPILER_COMPILATION_LEVEL]
        for absolute_path in self.absolute_paths:
            args.append('--js=%s' % absolute_path)
        handle, out = mkstemp()
        args.append('--js_output_file=%s' % out) # for some reason stdout stalls
        args = map(smart_str, args)
        print ('Compiling scripts in `%s` to `%s`' %
            (self.template, self.node.destination))
        p = Popen(args, stdout=PIPE)
        p.wait()
        with open(out, 'r') as fp:
            storage.save(self.node.destination, ContentFile(fp.read()))
        os.close(handle)
        os.remove(out)


class StyleCompressor(Shrink):
    already_treated = '(s)css files:\tup-to-date.'

    def therapy(self):
        css = []
        parser = Scss()
        for absolute_path in self.absolute_paths:
            with open(absolute_path, 'r') as fp:
                if absolute_path.endswith('.scss'):
                    css.append(parser.compile(fp.read()))
                else:
                    css.append(fp.read())
        handle, tmp = tempfile.mkstemp()
        with open(tmp, 'w') as fp:
            fp.write('\n'.join(css))
        args = ['java', '-jar', settings.SHRINK_YUI_COMPRESSOR,
                '--type', 'css', tmp]
        args = map(smart_str, args)
        print ('Compressing (s)css in `%s` to `%s`' %
            (self.template, self.node.destination))
        p = Popen(args, stdout=PIPE)
        p.wait()
        storage.save(self.node.destination, ContentFile(p.stdout.read()))
        os.close(handle)
        os.remove(tmp)




from django.core.cache import cache
from django.http import HttpResponse, Http404
from os.path import getmtime
from scss import Scss
from shrink.helpers import find_static


SCSS_CACHE_TIMEOUT = 3600


def scss(request, path):
    absolute_path = find_static(path)
    if not absolute_path:
        raise Http404("'%s' could not be found" % path)
    mtime = getmtime(absolute_path)
    cached = cache.get(path)
    if not cached or mtime > cached['mtime']:
        parser = Scss()
        with open(absolute_path, 'r') as fp:
            css = parser.compile(fp.read())
        response = HttpResponse(css, mimetype='text/css')
        cache.set(path, {
            'response': response,
            'mtime': mtime,
            }, SCSS_CACHE_TIMEOUT)
        return response
    return cached['response']

